@@ -1,20 +1,29 @@
+"""Use a network group and find its networks."""
+
 import json
+import logging
 
-from .utility import RESOURCES, STATUS_CODES, eprint, http
+from .utility import RESOURCES, STATUS_CODES, Utility, eprint, http, is_uuidv4
 
+utility = Utility()
 
 class NetworkGroup:
-    """use a Network Group by name or ID.
+    """use a network group by name or ID.
     
     The default is to use the first network group available to the organization of the caller.
     """
 
-    def __init__(self, Organization: object, network_group_id: str=None, network_group_name: str=None):
+    def __init__(self, Organization: object, network_group_id: str=None, network_group_name: str=None, group: str=None):
         """Initialize the network group class with a group name or ID."""
+        if (not network_group_id and not network_group_name) and group:
+            if is_uuidv4(group):
+                network_group_id = group
+            else:
+                network_group_name = group
         if network_group_id:
             self.network_group_id = network_group_id
             self.network_group_name = [ ng['organizationShortName'] for ng in Organization.network_groups if ng['id'] == network_group_id ][0]
-        # TODO: review the use of org short name ref https://netfoundry.slack.com/archives/C45UDKR8V/p1603655594135000?thread_ts=1580318187.149400&cid=C45UDKR8V
+        # TODO: review the use of org short name ref https://mattermost.tools.netfoundry.io/netfoundry/pl/gegyzuybypb9jxnrw1g1imjywh
         elif network_group_name:
             self.network_group_name = network_group_name
             network_group_matches = [ ng['id'] for ng in Organization.network_groups if ng['organizationShortName'] == network_group_name ]
@@ -23,38 +32,36 @@ class NetworkGroup:
             else:
                 raise Exception("ERROR: there was not exactly one network group matching the name \"{}\"".format(network_group_name))
         elif len(Organization.network_groups) > 0:
-            # first Network Group is typically the only Network Group
+            # first network group is typically the only network group
             self.network_group_id = Organization.network_groups[0]['id']
             self.network_group_name = Organization.network_groups[0]['organizationShortName']
             # warn if there are other groups
             if len(Organization.network_groups) > 1:
-                eprint("WARN: Using first Network Group {:s} and ignoring {:d} other(s) e.g. {:s}, etc...".format(
+                logging.warn("using first network group {:s} and ignoring {:d} other(s) e.g. {:s}, etc...".format(
                     self.network_group_name,
                     len(Organization.network_groups) - 1,
                     Organization.network_groups[1]['organizationShortName']
                 ))
             elif len(Organization.network_groups) == 1:
-                eprint("WARN: Using the default Network Group: {:s}".format(
+                logging.debug("using the only available network group: {:s}".format(
                     self.network_group_name
                 ))
         else:
-            raise Exception("ERROR: need at least one Network Group in organization")
+            raise Exception("need at least one network group in organization")
 
         self.session = Organization
         self.describe = Organization.get_network_group(self.network_group_id)
         self.id = self.network_group_id
         self.name = self.network_group_name
-        self.vanity = Organization.label.lower()
+        self.vanity = utility.normalize_caseless(Organization.label)
 
         if self.session.environment == "production":
             self.nfconsole = "https://{vanity}.nfconsole.io".format(vanity=self.vanity)
         else:
             self.nfconsole = "https://{vanity}.{env}-nfconsole.io".format(vanity=self.vanity, env=self.session.environment)
 
-    def nc_data_centers(self): # this was attribute self.nc_data_centers and converted to method to avoid calling preemptively with self.__init__
-        return(self.get_controller_data_centers())
-
-    def nc_data_centers_by_location(self): # this was attribute self.nc_data_centers_by_location and converted to method to avoid calling preemptively with self.__init__
+    def nc_data_centers_by_location(self):
+        """Get a controller data center by locationCode."""
         my_nc_data_centers_by_location = dict()
         for dc in self.get_controller_data_centers():
             my_nc_data_centers_by_location[dc['locationCode']] = dc['id']
@@ -62,15 +69,42 @@ class NetworkGroup:
         return(my_nc_data_centers_by_location)
 
     # resolve network UUIDs by name
-    def networks_by_name(self): # this was attribute self.networks_by_name and converted to method to avoid calling preemptively with self.__init__
+    def networks_by_name(self):
+        """Find networks in group by normalized name.
+        
+        Deprecated: use networks_by_normal_name
+        """
         my_networks_by_name = dict()
         for net in self.session.get_networks_by_group(self.network_group_id):
             my_networks_by_name[net['name']] = net['id']
         return(my_networks_by_name)
 
-    def get_controller_data_centers(self):
-        """list the data centers where a Network Controller may be created
+    def networks_by_normal_name(self):
+        """Find networks in group by case-insensitive (caseless, normalized) name.
+        
+        Case-insensitive uniqueness is enforced by the API for each type of entity.
         """
+        my_networks_by_normal_name = dict()
+        for name,id in self.networks_by_name().items():
+            my_networks_by_normal_name[utility.normalize_caseless(name)] = id
+        return(my_networks_by_normal_name)
+
+    def network_exists(self, name: str, deleted: bool=False):
+        """Check if a network exists.
+        
+        :param name: the case-insensitive string to search
+        :param deleted: include deleted networks in results
+        """
+        network_normal_names = list()
+        for net in self.session.get_networks_by_group(network_group_id=self.network_group_id, deleted=deleted):
+            network_normal_names.append(utility.normalize_caseless(net['name']))
+        if utility.normalize_caseless(name) in network_normal_names:
+            return(True)
+        else:
+            return(False)
+
+    def get_controller_data_centers(self):
+        """Find controller data centers."""
         try:
             # data centers returns a list of dicts (data center objects)
             headers = { "authorization": "Bearer " + self.session.token }
@@ -108,9 +142,11 @@ class NetworkGroup:
 
         return(aws_data_centers)
 
+    # provide a compatible alias
+    nc_data_centers = get_controller_data_centers
+
     def get_product_metadata(self, is_active: bool=True):
-        """fetch all product metadata
-        """
+        """Get all products' metadata."""
         try:
             response = http.get(
                 self.session.audience+'product-metadata/v2/download-urls.json',
@@ -146,8 +182,7 @@ class NetworkGroup:
             return (product_metadata)
 
     def list_product_versions(self, product_metadata: dict={}):
-        """find all product version from product metadata
-        """
+        """Find product versions in all products' metadata."""
         if product_metadata:
             product_versions = product_metadata.keys()
         else:
@@ -157,8 +192,7 @@ class NetworkGroup:
         return (product_versions)
 
     def find_latest_product_version(self, product_versions: list=[]):
-        """find the highest sorted product version number (may be experimental, not stable)
-        """
+        """Get the highest product version number (may be experimental, not stable)."""
         if not product_versions:
             product_versions = self.list_product_versions()
 
@@ -167,14 +201,14 @@ class NetworkGroup:
 
     def create_network(self, name: str, network_group_id: str=None, location: str="us-east-1", version: str=None, size: str="small"):
         """
-        create a network with
+        Create a network in this network group.
+
         :param name: required network name
-        :param network_group: optional Network Group ID
+        :param network_group: optional network group ID
         :param location: optional data center region name in which to create
         :param version: optional product version string like 7.3.17
         :param size: optional network configuration metadata name from /core/v2/network-configs e.g. "medium"
         """
-        
         my_nc_data_centers_by_location = self.nc_data_centers_by_location()
         if not location in my_nc_data_centers_by_location.keys():
             raise Exception("ERROR: unexpected Network location '{:s}'. Valid locations include: {}.".format(location, my_nc_data_centers_by_location.keys()))
@@ -243,19 +277,19 @@ class NetworkGroup:
 
     def delete_network(self, network_id=None, network_name=None):
         """
-        delete a Network
-        :param id: optional Network UUID to delete
-        :param name: optional Network name to delete
+        Delete a network.
+
+        :param id: optional network UUID to delete
+        :param name: optional network name to delete
         """
         try:
             networks_by_name = self.networks_by_name()
             if network_id:
-#                import epdb; epdb.serve()
                 network_name = next(name for name, uuid in networks_by_name.items() if uuid == network_id)
             elif network_name and network_name in networks_by_name.keys():
                 network_id = networks_by_name[network_name]
         except:
-            raise Exception("ERROR: need one of network_id or network_name for a Network in this Network Group: {:s}".format(self.name))
+            raise Exception("ERROR: need one of network_id or network_name for a Network in this network group: {:s}".format(self.name))
 
         try:
             headers = { "authorization": "Bearer " + self.session.token }
